<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;

/**
 * Observer on Adyen notification and set order status accordingly
 */
 
 class UpdateStatus implements ObserverInterface {
    
    /**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;
    
    const MANAGED_NOTIFICATION_CODES = array('AUTHORISATION', 'CAPTURE', 'MANUAL_REVIEW_ACCEPT');
    const WHITELISTED_METHODS = array('adyen_cc', 'adyen_oneclick');
    const WHITELISTED_METHOD_TYPES = array('bankTransfer_IBAN');
    const AUTHORIZED_ORDER_STATUS = 'confirmed';
    const CAPTURED_ORDER_STATUS = 'payment_confirmed';
    const DEBUG_LOG_ACTIVE = TRUE;
	
    /**
     * @param \Psr\Log\LoggerInterface $logger
     */  
	public function __construct(
		\Psr\Log\LoggerInterface $logger
	){
		$this->_logger = $logger;
	}

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer){
        /* @var $order \Magento\Sales\Model\Order */
        $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Notification");
        $notification_code = $observer->getEvent()->getNotificationCode();
        if (!in_array($notification_code, self::MANAGED_NOTIFICATION_CODES)) {
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Skipped notification");
            }
            return;
        }
        if ($notification_code == 'AUTHORISATION' && $observer->getEvent()->getManualReview()) {
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Skipped notification due to Manual review flag");
            }
            return;
        }
        $order = $observer->getEvent()->getOrder();
        $payment_method = $order->getPayment()->getMethod();
        $payment_method_type = $order->getPayment()->getPaymentMethodType();
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Order");
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: payment_method: $payment_method");
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: payment_method_type: $payment_method_type");
        }
        if (in_array($payment_method, self::WHITELISTED_METHODS)
                || in_array($payment_method_type, self::WHITELISTED_METHOD_TYPES)) {
            if ($order->hasInvoices()) {
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Order has invoices.");
                }
                $order->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), self::CAPTURED_ORDER_STATUS);
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Order status set to " . self::CAPTURED_ORDER_STATUS);
                }
            } else {
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Order has no invoices.");
                }
                $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING)
                        ->addStatusHistoryComment(
                    __('Order processed by mod2-order-status'), self::AUTHORIZED_ORDER_STATUS);
                if (self::DEBUG_LOG_ACTIVE) {
                    $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Order status set to " . self::AUTHORIZED_ORDER_STATUS);
                }
            }
        }
        return TRUE;
    }
}